﻿Type=Class
Version=3.2
@EndOfDesignText@
'Class module
Sub Class_Globals
	Dim Actor As lgScn2DActor
	Private Anim As lgAnimation
	Private argRightAnim(2), argLeftAnim(2) As lgTextureAtlasRegion
	Private argDownAnim(2), argUpAnim(2) As lgTextureAtlasRegion
	Private FrameDuration As Float = 0.3
	Private StateTime As Float
	Dim DirectionX, DirectionY As Byte
	Dim Speed As Int
End Sub

'Initializes the droid instance
Public Sub Initialize(Atlas As lgTextureAtlas)
	Actor.Initialize("PacDroid")
	argRightAnim = Atlas.FindRegions2("right")
	argDownAnim = Atlas.FindRegions2("down")
	For i = 0 To 1
		argLeftAnim(i).Initialize(argRightAnim(i))
		argLeftAnim(i).Flip(True, False)
		argUpAnim(i).Initialize(argDownAnim(i))
		argUpAnim(i).Flip(False, True)
	Next
	Actor.ScaleX = Main.Maze.TileSize / argRightAnim(0).OriginalWidth * 2
	Actor.ScaleY = Main.Maze.TileSize / argRightAnim(0).OriginalHeight * 2
	Actor.OriginX = argRightAnim(0).OriginalWidth / 2
	Actor.OriginY = argRightAnim(0).OriginalHeight / 2
End Sub

'Resets the droid data
Public Sub Reset
	Anim.Initialize(FrameDuration, argRightAnim)
	StateTime = 0
	DirectionX = Main.Direction_NONE
	DirectionY = Main.Direction_NONE
	Actor.ScaleX = Main.Maze.TileSize / argRightAnim(0).OriginalWidth * 2
	Actor.ScaleY = Main.Maze.TileSize / argRightAnim(0).OriginalHeight * 2
End Sub

'Moves horizontally the droid
Public Sub HorizontalMove
	If DirectionX = Main.Direction_RIGHT Then
		Dim Limit As Int = NextMove(Main.Maze.TileSize, 0)
		If Actor.X = Limit Then
			DirectionX = Main.Direction_NONE
		Else
			If Anim.GetKeyFrame(0) <> argRightAnim(0) Then
				Anim.Initialize(FrameDuration, argRightAnim)
			End If
			Actor.X = Min(Actor.X + Speed * Main.DeltaTime, Limit)
		End If
	Else If DirectionX = Main.Direction_LEFT Then
		Dim Limit As Int = NextMove(-Main.Maze.TileSize, 0)
		If Actor.X = Limit Then
			DirectionX = Main.Direction_NONE
		Else
			If Anim.GetKeyFrame(0) <> argLeftAnim(0) Then
				Anim.Initialize(FrameDuration, argLeftAnim)
			End If
			Actor.X = Max(Actor.X - Speed * Main.DeltaTime, Limit)
		End If
	End If
End Sub

'Moves vertically the droid
Public Sub VerticalMove
	If DirectionY = Main.Direction_TOP Then
		Dim Limit As Int = NextMove(0, Main.Maze.TileSize)
		If Actor.Y = Limit Then
			DirectionY = Main.Direction_NONE
		Else
			If Anim.GetKeyFrame(0) <> argUpAnim(0) Then
				Anim.Initialize(FrameDuration, argUpAnim)
			End If
			Actor.Y = Min(Actor.Y + Speed * Main.DeltaTime, Limit)
		End If
	Else If DirectionY = Main.Direction_BOTTOM Then
		Dim Limit As Int = NextMove(0, -Main.Maze.TileSize)
		If Actor.Y = Limit Then
			DirectionY = Main.Direction_NONE
		Else
			If Anim.GetKeyFrame(0) <> argDownAnim(0) Then
				Anim.Initialize(FrameDuration, argDownAnim)
			End If
			Actor.Y = Max(Actor.Y - Speed * Main.DeltaTime, Limit)
		End If
	End If
End Sub

Private Sub NextMove(DeltaX As Int, DeltaY As Int) As Int
	'Checks whether the droid can move to the next tile
	Dim NextTileX As Int = Main.Maze.FromPixelToTile_X(Actor.X + DeltaX)
	Dim NextTileY As Int = Main.Maze.FromPixelToTile_Y(Actor.Y + DeltaY)
	Dim Tile As lgMapStaticTiledMapTile = Main.Maze.GetTile(NextTileX, NextTileY)
	If Main.Maze.CanMoveTo(Tile) Then
		'Returns the next tile position
		If DeltaX <> 0 Then
			Return Main.Maze.FromTileToPixel_X(NextTileX)
		Else If DeltaY <> 0 Then
			Return Main.Maze.FromTileToPixel_Y(NextTileY)
		End If
	Else
		'Returns the current tile position
		If DeltaX <> 0 Then
			Return Main.Maze.FromTileToPixel_X(Main.Maze.FromPixelToTile_X(Actor.X))
		Else If DeltaY <> 0 Then
			Return Main.Maze.FromTileToPixel_Y(Main.Maze.FromPixelToTile_Y(Actor.Y))
		End If
	End If
	Return 0
End Sub

Private Sub PacDroid_Draw(SpriteBatch As lgSpriteBatch, ParentAlpha As Float)
	'Animates the droid
	Dim Frame As lgTextureAtlasRegion = Anim.GetKeyFrame2(StateTime, True)
	StateTime = StateTime + Main.DeltaTime
	SpriteBatch.SetColorRGBA(1, 1, 1, ParentAlpha)
	SpriteBatch.DrawRegion3(Frame, _
	                        Actor.X + (Frame.OffsetX * Actor.ScaleX) - Actor.OriginX + Main.Maze.TileSize, _
	                        Actor.Y + (Frame.OffsetY * Actor.ScaleY) - Actor.OriginY + Main.Maze.TileSize, _
	                        Actor.OriginX, Actor.OriginY, Frame.RegionWidth, Frame.RegionHeight, _
	                        Actor.ScaleX, Actor.ScaleY, 0)
End Sub
