﻿Type=Class
Version=3.2
@EndOfDesignText@
'Class module
Sub Class_Globals
	Dim Actor As lgScn2DImage
	Private Drawable As lgScn2DSpriteDrawable
	Private Index As Int
	Dim Mode, PreviousMode As Byte
	Dim ModeTime, PreviousModeTime As Float
	Dim ScatterCount As Byte
	Private ScatterTargetX, ScatterTargetY As Int
	Private TargetX, TargetY As Int
	Private MovingDirection As Byte
	Private CurrentTile As lgMapStaticTiledMapTile
	Type typMove(Possible As Boolean, Coordinates As lgMathVector2, Distance As Float)
End Sub

'Initializes the ghost instance
Public Sub Initialize(sprGhost As lgSprite, idxGhost As Int)
	Dim sdwGhost As lgScn2DSpriteDrawable
	sdwGhost.Initialize2(sprGhost)
	Actor.InitializeWithDrawable(sdwGhost, "")
	Drawable = sdwGhost
	Index = idxGhost
End Sub

'Sets the coordinates (in tiles) of the target in Scatter mode
Public Sub SetScatterTarget(TileX As Int, TileY As Int)
	ScatterTargetX = Main.Maze.FromTileToPixel_X(TileX)
	ScatterTargetY = Main.Maze.FromTileToPixel_Y(TileY)
End Sub

'Resets the ghost mode
Public Sub ResetMode
	ChangeMode(Main.Mode_SCATTER)
	ModeTime = 0
	ScatterCount = 3
End Sub

'Resets the ghost target
Public Sub ResetTarget
	MovingDirection = Main.Direction_NONE
	TargetX = -1
	TargetY = -1
End Sub

'Changes the behavior and appearance of the ghost
Public Sub ChangeMode(NewMode As Byte)
	If Mode <> NewMode Then
		Actor.ClearActions
		If NewMode = Main.Mode_FRIGHTENED Then
			'The ghost turns to blue
			PreviousMode = Mode
			PreviousModeTime = ModeTime
			Actor.Drawable = Main.ScaredGhost
		Else If NewMode = Main.Mode_RETURN Then
			'The ghost becomes translucent
			Actor.SetColorRGBA(0.5, 0.5, 0.5, 0.75)
		Else If Mode = Main.Mode_FRIGHTENED OR Mode = Main.Mode_RETURN Then
			'Back to the normal appearance
			Actor.Drawable = Drawable
			Actor.SetColorRGBA(1, 1, 1, 1)

			'Back to the previous mode
			Mode = PreviousMode
			ModeTime = PreviousModeTime
			Return
		End If
		Mode = NewMode
		ModeTime = 0
	End If
End Sub

Private Sub IsNotOccupied(TileX As Int, TileY As Int) As Boolean
	'Checks whether the given tile is not occupied by another ghost
	If Mode = Main.Mode_RETURN Then Return True 'No importance in Return mode
	For i = 0 To Main.Ghosts.Length - 1
		If i <> Index AND Main.Ghosts(i).Mode <> Main.Mode_RETURN Then
			Dim tX, tY As Int
			If Main.Ghosts(i).TargetX = -1 OR Main.Ghosts(i).TargetY = -1 Then
				tX = Main.Maze.FromPixelToTile_X(Main.Ghosts(i).Actor.X)
				tY = Main.Maze.FromPixelToTile_Y(Main.Ghosts(i).Actor.Y)
			Else
				tX = Main.Maze.FromPixelToTile_X(Main.Ghosts(i).TargetX)
				tY = Main.Maze.FromPixelToTile_Y(Main.Ghosts(i).TargetY)
			End If
			If tX = TileX AND tY = TileY Then
				'Already occupied
				Return False
			End If
		End If
	Next
	Return True
End Sub

Private Sub CanMoveTo(Tile As lgMapStaticTiledMapTile, IsInTheHouse As Boolean) As Boolean
	'Checks whether the given tile is not a wall
	If Tile = Null Then Return False 'Teleportation tile
	Return Main.Maze.CanMoveTo(Tile) OR ((IsInTheHouse OR Mode = Main.Mode_RETURN) _
	                                 AND (Tile = Main.Maze.RefDoorTile OR Tile = Main.Maze.RefHouseTile))
End Sub

Private Sub FindPossibleMoves As typMove(4)
	'Creates an array of possible moves with their future coordinates
	Dim Moves(4) As typMove
	Dim TileX As Int = Main.Maze.FromPixelToTile_X(Actor.X)
	Dim TileY As Int = Main.Maze.FromPixelToTile_Y(Actor.Y)
	CurrentTile = Main.Maze.GetTile(TileX, TileY)
	Dim IsInTheHouse As Boolean = (CurrentTile = Main.Maze.RefHouseTile)
	Moves(Main.Direction_RIGHT).Initialize
	If MovingDirection <> Main.Direction_LEFT Then
		'Checks the tile on the right
		Dim Tile As lgMapStaticTiledMapTile = Main.Maze.GetTile(TileX + 1, TileY)
		If CanMoveTo(Tile, IsInTheHouse) AND IsNotOccupied(TileX + 1, TileY) Then
			Moves(Main.Direction_RIGHT).Possible = True
			Moves(Main.Direction_RIGHT).Coordinates.Set(TileX + 1, TileY)
		End If
	End If
	Moves(Main.Direction_LEFT).Initialize
	If MovingDirection <> Main.Direction_RIGHT Then
		'Checks the tile on the left
		Dim Tile As lgMapStaticTiledMapTile = Main.Maze.GetTile(TileX - 1, TileY)
		If CanMoveTo(Tile, IsInTheHouse) AND IsNotOccupied(TileX - 1, TileY) Then
			Moves(Main.Direction_LEFT).Possible = True
			Moves(Main.Direction_LEFT).Coordinates.Set(TileX - 1, TileY)
		End If
	End If
	Moves(Main.Direction_TOP).Initialize
	If MovingDirection <> Main.Direction_BOTTOM Then
		'Checks the tile above
		Dim Tile As lgMapStaticTiledMapTile = Main.Maze.GetTile(TileX, TileY + 1)
		If CanMoveTo(Tile, IsInTheHouse) AND IsNotOccupied(TileX, TileY + 1) Then
			Moves(Main.Direction_TOP).Possible = True
			Moves(Main.Direction_TOP).Coordinates.Set(TileX, TileY + 1)
		End If
	End If
	Moves(Main.Direction_BOTTOM).Initialize
	If MovingDirection <> Main.Direction_TOP Then
		'Checks the tile below
		Dim Tile As lgMapStaticTiledMapTile = Main.Maze.GetTile(TileX, TileY - 1)
		If CanMoveTo(Tile, IsInTheHouse) AND IsNotOccupied(TileX, TileY - 1) Then
			Moves(Main.Direction_BOTTOM).Possible = True
			Moves(Main.Direction_BOTTOM).Coordinates.Set(TileX, TileY - 1)
		End If
	End If
	Return Moves
End Sub

Private Sub Move
	'Moves to the targeted tile
	If MovingDirection = Main.Direction_LEFT Then
		Actor.X = Max(Actor.X - Main.GhostSpeed * Main.DeltaTime, TargetX)
	Else If MovingDirection = Main.Direction_RIGHT Then
		Actor.X = Min(Actor.X + Main.GhostSpeed * Main.DeltaTime, TargetX)
	Else If MovingDirection = Main.Direction_BOTTOM Then
		Actor.Y = Max(Actor.Y - Main.GhostSpeed * Main.DeltaTime, TargetY)
	Else If MovingDirection = Main.Direction_TOP Then
		Actor.Y = Min(Actor.Y + Main.GhostSpeed * Main.DeltaTime, TargetY)
	End If
End Sub

Private Sub ContinueMove As Boolean
	If TargetX = -1 OR TargetY = -1 OR (Actor.X = TargetX AND Actor.Y = TargetY) Then
		'The targeted tile has been reached so the ghost can select a new tile
		Return False
	Else
		'Continues the move to the targeted tile
		Move
		Return True
	End If
End Sub

Private Sub MoveCloserTo(Position As lgMathVector2)
	'Is the current move finished?
	If ContinueMove Then Return

	'Creates an array of possible moves
	Dim Moves(4) As typMove = FindPossibleMoves
	If CurrentTile = Main.Maze.RefHouseTile Then
		'If the ghost is in the house, the outside area near the door replaces the given position
		Dim TileDoorX As Int = Main.Maze.CurrentLayer.Properties.Get("doorX")
		Dim TileDoorY As Int = Main.Maze.CurrentLayer.Properties.Get("doorY")
		If Main.Maze.FromPixelToTile_X(Actor.X) > TileDoorX Then TileDoorX = TileDoorX + 1
		Position.Set(Main.Maze.FromTileToPixel_X(TileDoorX), _
	   	          Main.Maze.FromTileToPixel_InvertedY(TileDoorY - 1))
	Else If CurrentTile = Main.Maze.RefTeleportTile Then
		'The ghost is moved to the other side of the tunnel
		Dim TileX As Int = Main.Maze.FromPixelToTile_X(Actor.X)
		If MovingDirection = Main.Direction_LEFT Then
			TileX = TileX + 26
			TargetX = Main.Maze.FromTileToPixel_X(TileX - 1)
		Else
			TileX = TileX - 26
			TargetX = Main.Maze.FromTileToPixel_X(TileX + 1)
		End If
		Actor.X = Main.Maze.FromTileToPixel_X(TileX)
		Move
		Return
	End If

	'Selects the shortest path between the ghost and the given position
	Dim Shortest_Idx As Int = -1
	For i = 0 To 3
		If Moves(i).Possible Then
			Moves(i).Coordinates.X = Main.Maze.FromTileToPixel_X(Moves(i).Coordinates.X)
			Moves(i).Coordinates.Y = Main.Maze.FromTileToPixel_Y(Moves(i).Coordinates.Y)
			Moves(i).Distance = Position.Dst2(Moves(i).Coordinates)
			If Shortest_Idx = -1 OR Moves(i).Distance < Moves(Shortest_Idx).Distance Then
				Shortest_Idx = i
			End If
		End If
	Next

	'Moves to the selected tile
	If Shortest_Idx > -1 Then
		MovingDirection = Shortest_Idx
		TargetX = Moves(Shortest_Idx).Coordinates.X
		TargetY = Moves(Shortest_Idx).Coordinates.Y
		Move
	Else
		'The ghost is blocked by another ghost so it is allowed to change direction
		MovingDirection = Main.Direction_NONE
	End If
End Sub

Public Sub MoveInChaseMode
	Dim PosDroid As lgMathVector2
	PosDroid.Set(Main.PacDroid.Actor.X, Main.PacDroid.Actor.Y)
	MoveCloserTo(PosDroid)
End Sub

Public Sub MoveInScatterMode
	Dim PosTarget As lgMathVector2
	PosTarget.Set(ScatterTargetX, ScatterTargetY)
	MoveCloserTo(PosTarget)
End Sub

Public Sub MoveInFrightenedMode
	'Is the current move finished?
	If ContinueMove Then Return

	'Creates an array of possible moves
	Dim Moves(4) As typMove = FindPossibleMoves
	If CurrentTile = Main.Maze.RefTeleportTile Then
		'The ghost is moved to the other side of the tunnel
		Dim TileX As Int = Main.Maze.FromPixelToTile_X(Actor.X)
		If MovingDirection = Main.Direction_LEFT Then
			TileX = TileX + 26
			TargetX = Main.Maze.FromTileToPixel_X(TileX - 1)
		Else
			TileX = TileX - 26
			TargetX = Main.Maze.FromTileToPixel_X(TileX + 1)
		End If
		Actor.X = Main.Maze.FromTileToPixel_X(TileX)
		Move
		Return
	End If

	'Selects the path that leads the ghost away from the droid
	Dim PosDroid As lgMathVector2
	PosDroid.Set(Main.PacDroid.Actor.X, Main.PacDroid.Actor.Y)
	Dim Farthest_Idx As Int = -1
	For i = 0 To 3
		If Moves(i).Possible Then
			Moves(i).Coordinates.X = Main.Maze.FromTileToPixel_X(Moves(i).Coordinates.X)
			Moves(i).Coordinates.Y = Main.Maze.FromTileToPixel_Y(Moves(i).Coordinates.Y)
			Moves(i).Distance = PosDroid.Dst2(Moves(i).Coordinates)
			If Farthest_Idx = -1 OR Moves(i).Distance > Moves(Farthest_Idx).Distance Then
				Farthest_Idx = i
			End If
		End If
	Next

	'Moves to the selected tile
	If Farthest_Idx > -1 Then
		MovingDirection = Farthest_Idx
		TargetX = Moves(Farthest_Idx).Coordinates.X
		TargetY = Moves(Farthest_Idx).Coordinates.Y
		Move
	Else
		'The ghost is blocked by another ghost so it is allowed to change direction
		MovingDirection = Main.Direction_NONE
	End If
End Sub

Public Sub MoveInReturnMode
	Dim TileDoorX As Int = Main.Maze.CurrentLayer.Properties.Get("doorX")
	Dim TileDoorY As Int = Main.Maze.CurrentLayer.Properties.Get("doorY")
	If Main.Maze.FromPixelToTile_X(Actor.X) > TileDoorX Then TileDoorX = TileDoorX + 1
	Dim PosHouse As lgMathVector2
	PosHouse.Set(Main.Maze.FromTileToPixel_X(TileDoorX), _
	             Main.Maze.FromTileToPixel_InvertedY(TileDoorY + 1))
	MoveCloserTo(PosHouse)
End Sub
