﻿Type=Class
Version=3.2
@EndOfDesignText@
'Class module
Sub Class_Globals
	Private Maps As lgMapTiledMap
	Private MapRenderer As lgMapOrthogonalTiledMapRenderer
	Dim CurrentLayer As lgMapTiledMapLayer
	Dim MapWidth, MapHeight As Int 'in tiles
	Dim TileSize As Float 'in pixels
	Dim RefFoodTile, RefEnergizerTile As lgMapStaticTiledMapTile
	Dim RefEmptyTile, RefTeleportTile As lgMapStaticTiledMapTile
	Dim RefHouseTile, RefDoorTile As lgMapStaticTiledMapTile
End Sub

'Loads the mazes and initializes the map renderer
Public Sub Initialize(Width As Int, Height As Int, GutterSize As Float, SpriteBatch As lgSpriteBatch)
	'Loads the mazes (TMX format)
	LoadMazes

	'Initializes the map renderer and computes the tile size
	MapWidth = Maps.Properties.Get("width")
	MapHeight = Maps.Properties.Get("height")
	Dim TileWidth As Int = Maps.Properties.Get("tilewidth")
	Dim TileHeight As Int = Maps.Properties.Get("tileheight")
	Dim RatioW As Float = Width / (TileWidth * MapWidth)
	Dim RatioH As Float = (Height - (GutterSize * 2)) / (TileHeight * MapHeight)
	If RatioW < RatioH Then
		MapRenderer.Initialize4(Maps, RatioW, SpriteBatch)
		TileSize = MapRenderer.UnitScale * TileWidth
	Else
		MapRenderer.Initialize4(Maps, RatioH, SpriteBatch)
		TileSize = MapRenderer.UnitScale * TileHeight
	End If

	'Selects the default layer
	SelectLayer(0)

	'Gets the reference tiles
	GetRefTiles
End Sub

Private Sub LoadMazes
	'Loads the mazes (TMX format)
	Dim TMXLoader As lgMapTmxMapLoader
	Maps = TMXLoader.Initialize2("maps/Pacdroid.tmx")
End Sub

Private Sub GetRefTiles
	'Gets the reference tiles
	RefEmptyTile = Maps.TileSets.GetTile(29)
	RefFoodTile = Maps.TileSets.GetTile(33)
	RefDoorTile = Maps.TileSets.GetTile(283)
	RefHouseTile = Maps.TileSets.GetTile(308)
	RefTeleportTile = Maps.TileSets.GetTile(399)
	RefEnergizerTile = Maps.TileSets.GetTile(426)
End Sub

'Reloads the mazes
Public Sub Reload
	Maps.dispose
	LoadMazes
	If MapRenderer.IsInitialized Then
		MapRenderer.Map = Maps
	Else
		MapRenderer.Initialize4(Maps, MapRenderer.UnitScale, MapRenderer.SpriteBatch)
	End If
	GetRefTiles
End Sub

'Each layer is a different maze
Public Sub SelectLayer(Layer As Int)
	CurrentLayer = Maps.Layers.Get(Layer)
End Sub

Public Sub FromTileToPixel_X(TileX As Int) As Int
	Return Round((TileX - 0.5) * TileSize)
End Sub

Public Sub FromTileToPixel_Y(TileY As Int) As Int
	Return Round((TileY - 0.5) * TileSize)
End Sub

Public Sub FromTileToPixel_InvertedY(TileY As Int) As Int
	Return Round((MapHeight - 1.5 - TileY) * TileSize)
End Sub

Public Sub FromPixelToTile_X(PixelX As Int) As Int
	Return Round((PixelX / TileSize) + 0.5)
End Sub

Public Sub FromPixelToTile_Y(PixelY As Int) As Int
	Return Round((PixelY / TileSize) + 0.5)
End Sub

'Checks whether the given tile is not a wall
Public Sub CanMoveTo(Tile As lgMapStaticTiledMapTile) As Boolean
	Return (Tile = RefFoodTile OR Tile = RefEnergizerTile OR _
	        Tile = RefEmptyTile OR Tile = RefTeleportTile)
End Sub

'Gets the tile at the given coordinates
Public Sub GetTile(TileX As Int, TileY As Int) As lgMapStaticTiledMapTile
	Dim Cell As lgMapTiledMapLayerCell = CurrentLayer.GetCell(TileX, TileY)
	If Cell = Null Then
		Return Null
	Else
		Return Cell.Tile
	End If
End Sub

'Draws the current layer
Public Sub Draw(Camera As lgOrthographicCamera)
	MapRenderer.SetCameraView(Camera)
	MapRenderer.RenderTileLayer(CurrentLayer)
End Sub

Public Sub Dispose
	Maps.dispose
	MapRenderer.dispose
End Sub
